package de.intevation.flys.artifacts.model;

import java.util.List;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;

import org.hibernate.Session;

import org.hibernate.SQLQuery;
import org.hibernate.type.StandardBasicTypes;

import de.intevation.flys.artifacts.cache.CacheFactory;

import de.intevation.flys.backend.SessionHolder;


/**
 * Factory to access ready-made WKms for other (than computed) 'kinds' of
 * WST-data.
 */
public class WKmsFactory
{
    /** Private logger to use here. */
    private static Logger log = Logger.getLogger(WKmsFactory.class);

    /** Query to get km and ws for wst_id and column_pos. */
    public static final String SQL_SELECT_WS =
        "SELECT km, w FROM wst_w_values " +
        "WHERE wst_id = :wst_id AND column_pos = :column_pos";

    /** Query to get name for wst_id and column_pos. */
    public static final String SQL_SELECT_NAME =
        "SELECT name " +
        "FROM wst_columns "+
        "WHERE wst_id = :wst_id AND position = :column_pos";

    /** Query to get name (description) for wst_id. */
    public static final String SQL_SELECT_WST_NAME =
        "SELECT description from wsts "+
        "WHERE id = :wst_id";


    private WKmsFactory() {
    }


    /**
     * Get WKms for given column and wst_id, caring about the cache.
     */
    public static WKms getWKms(int column, int wst_id) {
        log.debug("WKmsFactory.getWKms");
        Cache cache = CacheFactory.getCache(StaticWKmsCacheKey.CACHE_NAME);

        StaticWKmsCacheKey cacheKey;

        if (cache != null) {
            cacheKey = new StaticWKmsCacheKey(wst_id, column);
            Element element = cache.get(cacheKey);
            if (element != null) {
                log.debug("Got static wst values from cache");
                return (WKms)element.getValue();
            }
        }
        else {
            cacheKey = null;
        }

        WKms values = getWKmsUncached(column, wst_id);

        if (values != null && cacheKey != null) {
            log.debug("Store static wst values in cache.");
            Element element = new Element(cacheKey, values);
            cache.put(element);
        }
        return values;
    }

    /** Get name for a WKms. */
    public static String getWKmsName(int wst_id) {
        log.debug("WKmsFactory.getWKmsName wst_id/" + wst_id);

        String name = null;
        Session session = SessionHolder.HOLDER.get();

        SQLQuery nameQuery = session.createSQLQuery(SQL_SELECT_WST_NAME)
            .addScalar("description", StandardBasicTypes.STRING);
        nameQuery.setInteger("wst_id",     wst_id);

        List<String> names = nameQuery.list();
        if (names.size() >= 1) {
            name = names.get(0);
        }

        return name;
    }

    /** Get name for a WKms. */
    public static String getWKmsName(int column, int wst_id) {
        log.debug("WKmsFactory.getWKmsName c/" + column + ", wst_id/" + wst_id);

        String name = null;
        Session session = SessionHolder.HOLDER.get();

        SQLQuery nameQuery = session.createSQLQuery(SQL_SELECT_NAME)
            .addScalar("name", StandardBasicTypes.STRING);
        nameQuery.setInteger("wst_id",     wst_id);
        nameQuery.setInteger("column_pos", column);

        List<String> names = nameQuery.list();
        if (names.size() >= 1) {
            name = names.get(0);
        }

        return name;
    }


    /**
     * Get WKms from db.
     * @param column the position columns value
     * @param wst_id database id of the wst
     * @return according WKms.
     */
    public static WKms getWKmsUncached(int column, int wst_id) {

        if (log.isDebugEnabled()) {
            log.debug("WKmsFactory.getWKmsUncached c/" + column + ", wst_id/" + wst_id);
        }

        WKmsImpl wkms = new WKmsImpl(getWKmsName(column, wst_id));

        Session session = SessionHolder.HOLDER.get();
        SQLQuery sqlQuery = session.createSQLQuery(SQL_SELECT_WS)
            .addScalar("km", StandardBasicTypes.DOUBLE)
            .addScalar("w",  StandardBasicTypes.DOUBLE);
        sqlQuery.setInteger("wst_id",     wst_id);
        sqlQuery.setInteger("column_pos", column);

        List<Object []> results = sqlQuery.list();

        for (int i = 0, N = results.size(); i < N; i++) {
            Object[] row = results.get(i);
            wkms.add((Double) row[0], (Double) row[1]);
        }

        return wkms;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
